using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Transactions;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Collections;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.GS1;
using Produmex.Foundation.HWInterfacing.Tools;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.DataObjects;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Devices;
using Produmex.Sbo.Logex.Data.Generators.BatchNumberGenerators;
using Produmex.Sbo.Logex.Data.Generators.PackingController;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.Reports;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;
using Produmex.Sbo.Logex.LenL.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.LenL.BusinessObjects.Definitions.Tables;
using System.Timers;
using Produmex.Foundation.Convertors;
using Produmex.Sbo.Logex.Data.ViewObjects;

// PmxWorkflowExecutionTypes.SUB_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
    /// <summary>
    /// DeliveryValidationScriptFittinq
    /// </summary>
    public class WorkflowScript_DeliveryValidationScriptFittinq : WorkflowInstanceScriptBase
    {
        /// <summary>
        /// Definitions
        /// </summary>
        private static class Definitions
        {
            /// <summary>
            /// The time to show a message on the screen
            /// </summary>
            public const int MESSAGE_TIME = 3;

            /// <summary>
            /// The minimum weight
            /// </summary>
            public const decimal MINIMUM_WEIGHT = 0.1M;

            /// <summary>
            /// The tolerance percentage lower than X kg
            /// </summary>
            public const int TOLERANCE_PERCENTAGE_LOWER_THAN_X_KG = 5;


            /// <summary>
            /// The tolerance reference value kg
            /// </summary>
            public const decimal TOLERANCE_REFERENCE_VALUE_KG = 2;

            /// <summary>
            /// The tolerance absolute difference
            /// </summary>
            public const decimal TOLERANCE_ABSOLUTE_DIFFERENCE = 0.05M;

			/// <summary>
			/// The weighing room code
			/// </summary>
			public const string WEIGHING_ROOM = "weigh2";
        }


        // Input parameters
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

        private CultureInfo current_DefaultCultureInfo
        {
            get { return DefaultCultureInfo.Get(); }
        }

        private PmxOseCompany currentPmxOseCompany
        {
            get { return PmxOseCompany.Get(); }
        }


        private WorkflowHelper m_methodHelper;
        ISboProviderService m_sboProviderService;

        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);


	    private Foundation.HWInterfacing.Tools.ScaleClient m_scaleClient;
	    private System.Timers.Timer m_hwTimer;
	    private DateTime m_hwScaleLastCommTs;
	    private string m_hwEnvironment_LastExceptionMessage;
	    private decimal m_weightData;
	    private bool m_weightDataRetreived;
		#region Constructors

		public WorkflowScript_DeliveryValidationScriptFittinq(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
           

        }

        #endregion

        #region WorkflowInstanceScriptBase Members

        /// <summary>
        /// Executes this instance.
        /// </summary>
        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            m_sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            IBarcodeController barcodeController = null;

            m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
                delegate (PmxDbConnection conn, object[] parameters)
                {
                    PmxBarcodeProvider bcProv = new PmxBarcodeProvider(conn);
                    barcodeController = bcProv.GetBarcodeController();

                    return null;
                });

            m_methodHelper = new WorkflowHelper()
            {
                CultureInfo = this.DefaultCultureInfo.Get(),
                ShowScreen = session.ShowScreen,
                WaitForMessage = this.WaitForMessage,
                SboProviderService = m_sboProviderService,
                BarcodeController = barcodeController,
                DeviceID = session.DeviceInstanceId
            };


            string sscc = null;

            GotoResult back;
            List<GotoResult> next;
            GotoResult result = GotoResult.IdentifySSCC;

            while (true)
            {
                switch (result)
                {
                    case GotoResult.IdentifySSCC:
                        back = GotoResult.Exit;
                        next = new List<GotoResult> {back, GotoResult.IdentifySSCC, GotoResult.CheckDelivery};
                        result = IdentifySSCC(out sscc, back);
                        break;
                    case GotoResult.CheckDelivery:
                        back = GotoResult.IdentifySSCC;
                        next = new List<GotoResult> {back};
                        result = CheckDelivery(sscc, back);
                        break;                    
                    case GotoResult.Exit:
                    default:
                        return;
                }

                if (!next.Contains(result))
                    throw new Exception("Invalid next step");
            }


        }


        #endregion

        #region HELPERS


        private GotoResult CheckDelivery(string sscc, GotoResult backGotoResult)
        {

            string query = BuildQuery.GetDeliveryForSSCC(sscc, m_methodHelper.SboProviderService.GetDbTool());
            DataSet dsDelivery = m_methodHelper.SboProviderService.RunView(false, null, null, query);

			if (GetNumberOfRows(dsDelivery) == 0)
			{
				m_methodHelper.InitialErrorKey = "MSG_ERROR_F_BOX_NOT_READY_FOR_VALIDATION";
				return GotoResult.IdentifySSCC;
			}

			if (!UpdateSalesDelivery(Convert.ToInt32(dsDelivery.Tables[0].Rows[0][PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.SalesDeliveryNoteLineDocEntry.NAME])))
			{
				m_methodHelper.InitialErrorKey = "MSG_ERROR_F_BOX_NOT_READY_FOR_VALIDATION";
				return GotoResult.IdentifySSCC;
			}

			//Update of sales delivery was succesful, so proceed to get weight
			decimal weightFromScale = GetWeightInKgFromScale(m_methodHelper, Definitions.WEIGHING_ROOM);
            if (weightFromScale < Definitions.MINIMUM_WEIGHT)
            {
                //Nothing has been put on the scale yet
                m_methodHelper.InitialErrorKey = "MSG_ERROR_F_NOTHING_ON_SCALE_YET";
                return GotoResult.IdentifySSCC;
            }

            decimal weightSSCC = Convert.ToDecimal(dsDelivery.Tables[0].Rows[0][PmxPickingLogisticUnitTable.Columns.PackageWeight.NAME]);

            if (IsWeightWithinTolerance(weightFromScale, weightSSCC))
            {
                //Within tolerance, so show message
                //show processing screen
                m_methodHelper.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                    DefaultCultureInfo.Get(),
                    BuildParamCollection(
                        "MessageKey", "MSG_F_BOX_OK",
                        "MaxTimeForMessage", Definitions.MESSAGE_TIME,
                        "IsError", false,
                        "ShowButton", true));
                m_methodHelper.WaitForMessage();
                return GotoResult.IdentifySSCC;
            }
            else
            {
                //Not within tolerance, show error
                m_methodHelper.InitialErrorKey = "MSG_ERROR_F_WEIGHT_NOT_WITHIN_TOLERANCE";
                return GotoResult.IdentifySSCC;
            }
        }

        private bool IsWeightWithinTolerance(decimal weightFromScale, decimal weightSSCC)
        {
            //Tolerance based on absolute value
            decimal difference = Definitions.TOLERANCE_ABSOLUTE_DIFFERENCE;

            if (weightSSCC < Definitions.TOLERANCE_REFERENCE_VALUE_KG)
            {
                //Tolerance based on percentage
                difference = weightSSCC * Definitions.TOLERANCE_PERCENTAGE_LOWER_THAN_X_KG / 100;
            }

            if (weightFromScale < weightSSCC - difference ||
                weightFromScale > weightSSCC + difference)
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        private decimal GetWeightInKgFromScale(WorkflowHelper methodHelper, string currentDockCode)
        {
	        WeighData weightData = new WeighData();

			weightData.LocationCode = currentDockCode;

			PmxDictionary<int, double> uomCoefficients = new PmxDictionary<int, double>();

			GetScaleData(methodHelper.SboProviderService, weightData, uomCoefficients);

			if (weightData.ScaleData != null)
			{
				if (!weightData.ScaleCoefficient.HasValue)
				{
					s_log.Warn(string.Format("There is no possible conversion between {0} and {1}", "KG", weightData.ScaleUom));
					weightData.ScaleCoefficient = 1.0;
				}

				PmxDictionary<string, string> scaleCommands = new PmxDictionary<string, string>();

				m_scaleClient = new Foundation.HWInterfacing.Tools.ScaleClient();

				foreach (PmxOseScaleCommand scaleCommand in weightData.ScaleData.ScaleCommands)
				{
					switch (scaleCommand.CommandType)
					{
						case PmxScaleCommandType.GetWeight:
							scaleCommands.Add("WeighCommand", scaleCommand.CommandFormat);
							break;
						case PmxScaleCommandType.SetTare:
							scaleCommands.Add("TareCommand", scaleCommand.CommandFormat);
							break;
						case PmxScaleCommandType.SetZero:
							scaleCommands.Add("ZeroCommand", scaleCommand.CommandFormat);
							break;
						case PmxScaleCommandType.WeightReturn:
							scaleCommands.Add("WeighResultFormat", scaleCommand.CommandFormat);
							break;
						case PmxScaleCommandType.TareReturn:
							break;
						case PmxScaleCommandType.ZeroReturn:
							break;
						default:
							break;
					}
				}

				try
				{
					m_weightData = 0;
					m_weightDataRetreived = false;
					m_scaleClient.Open(weightData.ScaleData.UseAsService ? "Service" : "GenericScale", weightData.ScaleData.ScaleDefinition, scaleCommands);

					if (m_scaleClient.Environment != null)
					{
						s_log.Info(string.Format("HW Environment found"));

						m_scaleClient.Client.OnMsgReceived += new Foundation.HWInterfacing.Clients.InternalClient.MsgReceivedDelegate(Client_OnMsgReceived);

						EnableScale(true);

						m_hwTimer = new System.Timers.Timer();
						m_hwTimer.Interval = 100;
						m_hwTimer.Elapsed += new ElapsedEventHandler(m_hwTimer_Tick);
						m_hwTimer.Start();
					}


					methodHelper.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						methodHelper.CultureInfo, BuildParamCollection(
						"InitialErrorKey", methodHelper.InitialErrorKey,
						"MessageKey", "Getting weigh from scale",
						"ShowButton", false));

					DateTime weighStart = DateTime.Now;

					while (!m_weightDataRetreived && (DateTime.Now - weighStart).TotalSeconds < 2.0)
					{
						Thread.Sleep(200);
					}
				}
				finally
				{
					m_hwTimer.Stop();
					m_scaleClient.Close();
				}
			}
			return m_weightData;
        }

		private void EnableScale(bool enable)
		{
			m_hwScaleLastCommTs = DateTime.Now;

			if (m_scaleClient.Client != null)
			{

				Foundation.HWInterfacing.Data.IDataBlock data = m_scaleClient.Client.DataParser.GetNewDataBlock("dev");
				data.SetAttribute("name", m_scaleClient.ScaleDeviceName);

				if (!String.IsNullOrEmpty(m_scaleClient.ScaleSubUnit))
				{
					data.SetAttribute("sub-unit", m_scaleClient.ScaleSubUnit);
				}

				data.SetAttribute("cmd", "cur-weight");
				data.SetAttribute("count", enable ? "0" : "-1");
				data.SetAttribute("delay", "300");
				m_scaleClient.Client.ProcessCommand(data);
			}
		}

		void m_hwTimer_Tick(object sender, EventArgs e)
		{
			try
			{
				m_scaleClient.Environment.Process();

				if ((DateTime.Now - m_hwScaleLastCommTs).TotalSeconds >= 1.0)
				{
					EnableScale(true);
				}
			}
			catch (Exception ex)
			{
				if (ex.Message != m_hwEnvironment_LastExceptionMessage)
				{
					m_hwEnvironment_LastExceptionMessage = ex.Message;

					s_log.Error("", ex);
				}
			}
		}

		void Client_OnMsgReceived(Foundation.HWInterfacing.Clients.InternalClient sender, Foundation.HWInterfacing.Data.IDataBlock data)
		{
			try
			{
				s_log.Debug(string.Format("Scale package type: {0}", data.GetName()));
			}
			catch { }
			try
			{
				s_log.Debug(string.Format("Scale name: {0}", data.GetAttribute("name")));
			}
			catch { }
			try
			{
				s_log.Debug(string.Format("Local scale name: {0}", m_scaleClient.ScaleDeviceName));
			}
			catch { }
			try
			{
				s_log.Debug(string.Format("Command name: {0}", data.GetAttribute("cmd")));
			}
			catch { }
			try
			{
				s_log.Debug(string.Format("Error code: {0}", data.GetAttribute("error")));
			}
			catch { }
			try
			{
				s_log.Debug(string.Format("Weight: {0}", data.GetAttribute("net")));
			}
			catch { }
			switch (data.GetName())
			{
				case "dev":
					if (data.GetAttribute("name") == m_scaleClient.ScaleDeviceName)  // Only act on messages regarding our scale-device
					{
						string cmd = data.GetAttribute("cmd");
						int error = SystemInt32Convertor.ObjectToInt(data.GetAttribute("error"), 0, 0);
						switch (cmd)
						{
							case "event:cur-weight":
								if (error == 0)
								{
									// Current weight has been retrieved successfully
									m_hwScaleLastCommTs = DateTime.Now;
									decimal net;
									if (decimal.TryParse(data.GetAttribute("net"), NumberStyles.Float, CultureInfo.InvariantCulture, out net))
									{
										this.m_weightDataRetreived = true;
										this.m_weightData = net;
									}
								}
								break;
						}
					}
					break;
			}
		}

		private bool UpdateSalesDelivery(int salesDeliveryDocEntry)
        {
            while (true)
            {
                try
                {
                    bool retVal = m_methodHelper.SboProviderService.InvokeMethodWithDbConnection<bool>(true, false, null, null,
                        delegate(PmxDbConnection conn, object[] parameters)
                        {
                            PmxSalesDeliveryNoteProvider dlnProv = new PmxSalesDeliveryNoteProvider((SboConnection) conn);
                            using (PmxSalesDeliveryNote dln = dlnProv.GetBO(salesDeliveryDocEntry))
                            {
                                if (dln == null)
                                {
                                    //Sales delivery not found, so no update possible
                                    return false;
                                }

                                dln[PmxSalesDeliveryNoteTableLenL.Columns.DeliveryDevice.NAME] = m_methodHelper.DeviceID;

                                dlnProv.UpdateBO(dln, false, false);

                                return true;
                            }
                        });

                    return retVal;

                }
                catch (Exception ex)
                {
                    s_log.Error(ex);
                    ShowMessageScreenForException(ex, "ShowButton", true);

                    //retry?
                    //ask user if he wants to retry
                    m_methodHelper.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                        m_methodHelper.CultureInfo, BuildParamCollection(
                            "TitleKey", "MSG_TITLE_RETRY_UPDATE",
                            "MessageKey", "MSG_QUESTION_AN_ERROR_OCCURED_WANT_RETRY"));
                    m_methodHelper.Msg = m_methodHelper.WaitForMessage();

                    if (m_methodHelper.Msg.Name.ToLower().EndsWith(".no"))
                    {
                        break;
                    }
                }
            }

            return true;

        }


        private void ShowMessageScreenForException(Exception ex, params object[] parameters)
        {
            if (!(ex is ProdumexException) || ((ProdumexException) ex).Error == null)
            {
                m_methodHelper.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                    m_methodHelper.CultureInfo,
                    BuildParamCollection("MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
                        "NoTranslationOfMessageKey", true,
                        "IsError", true, 
                        parameters));
            }
            else
            {
                m_methodHelper.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                    m_methodHelper.CultureInfo,
                    BuildParamCollection("PmxError", ((ProdumexException)ex).Error, parameters));
            }

           

            m_methodHelper.Msg = m_methodHelper.WaitForMessage();
        }

        private GotoResult IdentifySSCC(out string sscc, GotoResult backGotoResult)
        {
            sscc = null;


            m_methodHelper.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifyGS1DataScreen),
                m_methodHelper.CultureInfo, BuildParamCollection(
                    "InitialErrorKey", m_methodHelper.InitialErrorKey,
                    "TitleKey", "MSG_TITLE_IDENTIFY_SSCC",
                    "BarcodeController", m_methodHelper.BarcodeController,
                    "ForceGS1_128", true,
                    "ImageType", GS1ImageType.PalletImage
                ));

            m_methodHelper.Msg = m_methodHelper.WaitForMessage();
            m_methodHelper.InitialErrorKey = null;


            if (m_methodHelper.Msg.Name.EndsWith(".DataEntered"))
            {
                BarcodeContainer barcodeContainerSSCC = ExtractParameter<BarcodeContainer>(m_methodHelper.Msg.Parameters, "data");
                //check what has been scanned

                if (barcodeContainerSSCC.GS1DataObject != null && !string.IsNullOrEmpty(barcodeContainerSSCC.GS1DataObject.Sscc))
                {
                    //user scanned an SSCC, so we check if it is allowed
                    sscc = barcodeContainerSSCC.GS1DataObject.Sscc;
                    return GotoResult.CheckDelivery;
                }


                m_methodHelper.InitialErrorKey = "MSG_ERROR_BARCODE_DOES_NOT_CONTAIN_SSCC";
                return GotoResult.IdentifySSCC;
            }
            else if (m_methodHelper.Msg.Name.EndsWith(".ButtonClicked"))
            {
                //should not happen
                m_methodHelper.InitialErrorKey = "MSG_ERROR_OPERATION_NOT_SUPPORTED";
                return GotoResult.IdentifySSCC;
            }

            //back requested
            return backGotoResult;
        }

		private void GetScaleData(ISboProviderService sboProviderService, WeighData weightData, PmxDictionary<int, double> uomCoefficients)
		{
			weightData.ScaleData = sboProviderService.InvokeMethodWithDbConnection<PmxOseScale>(true, false, null, null,
						delegate (PmxDbConnection conn, object[] parameters)
						{
							SboUomProvider prov = new SboUomProvider((SboConnection)conn);							

							PmxOseScaleProvider scaleProvider = new PmxOseScaleProvider(conn);
							PmxOseScale scale = null;

							scale = scaleProvider.GetScaleForLocationWithUomAndQuantity(weightData.ScaleCode, weightData.LocationCode, uomCoefficients, 0, weightData.Quantity, 1000000);

							if (scale != null)
							{
								weightData.ScaleCoefficient = prov.GetCoefficientForUom(scale.UomEntry, 1000000, uomCoefficients);
								weightData.ScaleUom = prov.GetUomCodeOfUomEntry(scale.UomEntry);
								weightData.ScaleCode = scale.Code;

								NumberFormatInfo nfi = new NumberFormatInfo();
								nfi.NumberDecimalSeparator = ".";

								string[] accuracyComponents = scale.Accuracy.ToString(nfi).Split('.');

								weightData.ScaleUomDecimals = accuracyComponents.Length == 2 ? accuracyComponents[1].Length : 0;								
							}
							return scale;
						});
		}

		/// <summary>
		/// Gets the number of rows.
		/// </summary>
		/// <param name="ds">The ds.</param>
		/// <returns></returns>
		private static int GetNumberOfRows(DataSet ds)
        {
            if (ds != null &&
                ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }

            return 0;
        }


        private class WorkflowHelper
        {
            public ISboProviderService SboProviderService;
            public Action<Type, CultureInfo, ReadOnlyCollection<Parameter>> ShowScreen;
            public Func<Message> WaitForMessage;
            public CultureInfo CultureInfo;
            public Message Msg;
            public string InitialErrorKey;
            public IBarcodeController BarcodeController;
            public string DeviceID;
        }

	    internal class WeighData
	    {
		    public PmxOseScale ScaleData;
		    public string ScaleCode;
		    public string ScaleUom;
		    public double? ScaleCoefficient;
		    public string ItemUom;
		    public double PlannedQuantity;
		    public int ItemUomDecimals;
		    internal string TareCommand;
		    internal string ZeroCommand;
		    internal string WeighResultFormat;
		    internal string WeighCommand;
		    internal int ScaleUomDecimals;
		    internal string LocationCode;
		    internal double Quantity;
	    }

		private enum GotoResult
        {
            Exit,
            IdentifySSCC,
            CheckDelivery
        }

        #endregion

        #region BUILD QUERY

        /// <summary>
        /// Static class grouping our native queries
        /// </summary>
        internal static class BuildQuery
        {
	        public static string GetConversionToMiligrams(int uom, DbTool dbTool)
	        {
		        DbQueryBuilder qb = dbTool.GetQueryBuilder();

		        qb = qb.Select.
			        EscTableColumnName(SboUomTable.NAME, "Weight1").
			        Append(" * ").
			        EscTableColumnName(SboWeightUnitTable.NAME, SboWeightUnitTable.Columns.WeightInMG.NAME).
			        From.
			        EscTableName(SboUomTable.NAME).
			        CRLF.
			        LeftJoin.
			        EscTableName(SboWeightUnitTable.NAME).On.
			        EscTableColumnName(SboWeightUnitTable.NAME, SboWeightUnitTable.Columns.UnitCode.NAME).Eq.
			        EscTableColumnName(SboUomTable.NAME, "WghtUnit").
			        CRLF.
			        Where.
			        AppendFormat(SqlCommandHelper.FormatAndEscapeColumnValue(uom, dbTool.CultureInfo)).Eq.
			        EscTableColumnName(SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);

		        //  select OUOM.Weight1 * "OWGT"."WightInMG" from OUOM left join owgt on OWGT.UnitCode = ouom.WghtUnit where ouom.UomEntry = ?

		        return qb.ToString();
	        }

			internal static string GetDeliveryForSSCC(string sscc, DbTool dbTool)
            {
                /*
                 select TOP(1) PMX_DLPL.SalesDeliveryNoteLineDocEntry, PMX_PILU.PackageWeight, PMX_PILU.WeightUom FROM
PMX_DLPL 
INNER JOIN PMX_MVLI ON PMX_MVLI.DocEntry = PMX_DLPL.MoveLineDocEntry
AND PMX_MVLI.LineNum = PMX_DLPL.MoveLineLineNum
INNER JOIN PMX_LUID ON PMX_LUID.InternalKey = PMX_MVLI.DestLogUnitIdentKey
INNER JOIN PMX_PILU ON PMX_PILU.DocEntry = PMX_DLPL.PickListLineDocEntry
AND PMX_PILU.ObjectType = 'PMX_PLHE'
WHERE PMX_LUID.SSCC = '123456789012345678'
AND PMX_DLPL.CreateDate > (GetDate()-365)
ORDER BY PMX_DLPL.CreateDate

                 */
                DbQueryBuilder qb = dbTool.GetQueryBuilder();

                qb = qb.SelectTop1.
	                EscTableColumnNameWithAlias(PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME, PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.SalesDeliveryNoteLineDocEntry.NAME, PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.SalesDeliveryNoteLineDocEntry.NAME).Comma.
	                EscTableColumnNameWithAlias(PmxPickingLogisticUnitTable.NAME, PmxPickingLogisticUnitTable.Columns.PackageWeight.NAME, PmxPickingLogisticUnitTable.Columns.PackageWeight.NAME).Comma.
					EscTableColumnNameWithAlias(PmxPickingLogisticUnitTable.NAME, PmxPickingLogisticUnitTable.Columns.PackageWeight.NAME, PmxPickingLogisticUnitTable.Columns.LogisticCarrierCode.NAME).Comma.
                    EscTableColumnNameWithAlias(PmxPickingLogisticUnitTable.NAME, PmxPickingLogisticUnitTable.Columns.WeightUom.NAME, PmxPickingLogisticUnitTable.Columns.WeightUom.NAME).
                    CRLF.
                From.CRLF.
                    EscTableName(PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME).WithNoLock.
                    CRLF.
                InnerJoin.
                    EscTableName(PmxMoveLineTable.NAME).WithNoLock.On.
                    EscTableColumnName(PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DocEntry.NAME).Eq.
                    EscTableColumnName(PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME, PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.MoveLineDocEntry.NAME).
                    And.EscTableColumnName(PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.LineNum.NAME).Eq.
                    EscTableColumnName(PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME, PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.MoveLineLineNum.NAME).
                    CRLF.
                InnerJoin.
                    EscTableName(PmxLogisticUnitIDTable.NAME).WithNoLock.On.
                    EscTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME).Eq.
                    EscTableColumnName(PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME).
                    CRLF.
                InnerJoin.
                    EscTableName(PmxPickingLogisticUnitTable.NAME).WithNoLock.On.
                    EscTableColumnName(PmxPickingLogisticUnitTable.NAME, PmxPickingLogisticUnitTable.Columns.DocEntry.NAME).Eq.
                    EscTableColumnName(PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME, PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.PickListLineDocEntry.NAME).
                    And.EscTableColumnName(PmxPickingLogisticUnitTable.NAME, PmxPickingLogisticUnitTable.Columns.ObjectType.NAME).Eq.
                    AppendFormat(SqlCommandHelper.FormatAndEscapeColumnValue(Data.BusinessObjects.Definitions.PmxObjectTypes.PmxPickList, dbTool.CultureInfo)).
                    CRLF.
                Where.
                    EscTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME).Eq.
                    AppendFormat(SqlCommandHelper.FormatAndEscapeColumnValue(sscc, dbTool.CultureInfo)).
                    And.
                    CRLF.
                    EscTableColumnName(PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME, PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.CreateDate.NAME).
                    //Get all deliveries from max 1 year ago
                    Append(" > (GetDate()-365) ").
                    CRLF.
                OrderBy.
                    EscTableColumnName(PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME, PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.CreateDate.NAME).Desc;

                return qb.ToString();
            

            }
        }

        #endregion

    }


}
